/***************************************************************************

Name:			Morse Code Sending Library Demo Driver
Date:			2008.03.16
Author:			GJO
Version:		0.00

Description:	This library is designed to function in the background
				as a state machine while the processor executes
				unrelated other tasks. It requires the use of a 16-bit
				timer to generate the time base for the morse code.
				The CW sending uses a buffered character storage
				interface to allow for perfect timing.

Credits: 		Basic CW sending code and morse code encoding adapted from
				the PIC Beacon Keyer program by
				Wolfgang Buescher (DL4YHF), Spenge, Germany
				to function as a state machine based background task.

Setup:			morse_initialize(); must be called from main() first

USAGE:			put_morse(c);

				or: printf(put_morse, "....");
				(Standard fprint formatting still works here)

Examples:		printf(put_morse, "CQ CQ CQ DE HAMRADIO");
				printf(put_morse, "%03.1f DEG C", ftempC);
				printf(put_morse, "%02x", u8_number); (etc)

				fputc(c, put_morse);

Morse speed:	The morse speed setting is determined by the overflow
				value of the timer load value:

				ex: 	set_timer3(40536);

WPM:			To calculate the value for words per minute, use this
				equation:

				T3[us] = 65536 - (3000000 / (WPM * 10))

				Note: Normally fixed point is used. For example to send at
						10WPM use 100; WPM is 100 / 10

				Use calc_WordsPerMinute(long temp); to calculate the correct
				TIMER re-load value.

				The re-load is stored in global variable "_morse_speed".

Buffer Size:	TX_BUFFER_SIZE - Set to whatever size that is resonable

Compatible
Target:			Any 16F or 18F PIC with enough FlashROM and a 16-bit timer
				(Port to AVR is planned)

TX Key output:	Assign to any desired pin available. Re-define TX_PTT in
				Beacon.h file.

Compiler used:	CCS PCH V4.069
MPU:			PIC18F4620
Programmer:		Quick-Writer from Tech-Tools

Revision List:

	Ver     Date        Who     Description
	---------------------------------------------------------------------
    1.00    06.12.15    GJO     Wrote program for CW library


***************************************************************************/

#include "18F4620.h"

#use delay(clock=16000000, RESTART_WDT)
#fuses HS, NOPROTECT, NOPUT, BROWNOUT, NOLVP
#use rs232(baud=57600,xmit=PIN_C6,rcv=PIN_C7, enable=PIN_A5)

#include "I2CLCD_BEACON.c"	// CW Library Module


void main(void)
{
	int i;

	morse_initialize();

	calc_WordsPerMinute(150);	// 15 WPM

	enable_interrupts(GLOBAL);

	delay_ms(1000);

	printf("\r\nCW Beacon Demo Program\r\n"); // To RS232 port

	printf(put_morse, "HELLO WORLD"); // To Morse code output

	// Fill Morse buffer with ASCII characters from ' ' to Z
	for(i=32;i<=90;i++)
	{
		printf(put_morse,"%c", i);
	}

	printf("Filling Buffer complete\r\n"); // To RS232
	printf("Program main code is now waiting for code to finish\r\n"); // To RS232

	while(CW_send_active);	// Wait here until finish sending

	printf("Done Sending morse code\r\n");

	put_morse('s'); // Send the 'SK' proword to sign off

	for(;;); // Waiting forever while interrupt continues to send morse code
}







