/***************************************************************************

Name:			Morse Code Sending Library using background mode
Date:			2008.03.16
Author:			GJO
Version:		0.00

Description:	This library is designed to function in the background
				as a state machine while the processor executes
				unrelated other tasks. It requires the use of a 16-bit
				timer to generate the time base for the morse code.
				The CW sending uses a buffered character storage
				interface to allow perfect timing.

Credits: 		CW sending code and morse code encoding adapted from
				the PIC Beacon Keyer program by
				Wolfgang Buescher (DL4YHF), Spenge, Germany
				to function as a state machine based background task.

Setup:			morse_initialize(); must be called from main() first

USAGE:			cw put character wrapper:	put_morse(c)

				or: printf(put_morse, "....");
				(Standard fprint formatting still works here)

Examples:		printf(put_morse, "CQ CQ CQ DE HAMRADIO");
				printf(put_morse, "%03.1f DEG C", ftempC);
				printf(put_morse, "%02x", u8_number); (etc)

				put_morse('s');

Morse speed:	The morse speed setting is determined by the overflow
				value of the timer load value:

				ex: 	set_timer3(40536);

WPM:			To calculate the value for words per minute, use this
				equation:

				T3[us] = 65536 - (3000000 / (WPM * 10))

				Note: Normally fixed point is used. For example to send at
						10WPM use 100; WPM is 100 / 10

				Use calc_WordsPerMinute(long temp); to calculate the correct
				TIMER re-load value.

				The re-load is stored in global variable "_morse_speed".

Buffer Size:	TX_BUFFER_SIZE - Set to whatever size that is resonable

Compatible
Target:			Any 16F or 18F PIC with enough FlashROM and a 16-bit timer
				(Port to AVR is planned)

TX Key output:	Assign to any desired pin available. Re-define TX_PTT in
				Beacon.h file.

RS232 Debug:	uncomment #define MORSE_DEBUG to send output to RS232 port

Character Set:	Use ASCII 32 to 90 range only. Specified lower case characters
				are used for special Morse code proword signals.

Compiler used:	CCS PCH V4.069
MPU:			PIC18F4620
Programmer:		Quick-Writer from Tech-Tools

Revision List:

	Ver     Date        Who     Description
	---------------------------------------------------------------------
    1.00    06.12.15    GJO     Wrote program for CW library


***************************************************************************/
//#define MORSE_DEBUG
#include "I2CLCD_BEACON.h"


#int_timer2
void Audio_isr (void)
{
	if(fTone_send)
		output_toggle(TX_PTT);
	else
		output_bit(TX_PTT, NO_TONE);

}


// Setup TIMER3 hardware
void morse_initialize(void)
{
	// Clock for Morsecode timing
	setup_timer_2(T2_DIV_BY_16, 124, 1);	// 2KHZ
	output_bit(TX_PTT, NO_TONE);
	setup_timer_3(T3_INTERNAL | T3_DIV_BY_8);
	set_timer3(MORSE_SPEED_DEFAULT);
	enable_interrupts(INT_TIMER2);
	enable_interrupts(INT_TIMER3);
}


// Calculate Timer re-load value based on WPM * 10
void calc_WordsPerMinute(long temp)
{
	if(temp > 300) temp = 300; // 30 WPM
	if(temp < 50) temp = 50;   //  5 WPM

	// Calculate T3 timer overflow intervall calibrated in WPM
	// T3[us] = 65536 - (3000000 / (WPM * 10))
	// WPM is stored in WPM*10 Fixed Point format

    _morse_speed = (long) (65536 - (3000000 / (int32) temp));
}


// Write a character to the morse buffer handler
void put_morse(char c)
{
	char stuffit = 0;

	while(TX_Counter > (TX_BUFFER_SIZE-1))	// Wait!! Buffer is getting full
			;
	// if buffer empty, setup for interrupt
	if(TX_Counter == 0)
		stuffit = 1;
	// jam the character into the buffer
	TX_Buffer[TX_Wr_Index++] = c;

	// wrap the pointer
	if(TX_Wr_Index > TX_BUFFER_SIZE)
		TX_Wr_Index = 0;
	// keep track of the buffered characters
	TX_Counter++;

	// Kickstart the sending process
	if(stuffit == 1)
	{
		fPrimeIt = 1;
		TX_morse_char = c;
		morse_state = MORSE_SEND;
	}
}


// Morse isr to generate the morse code
#int_timer3
void morsecode_isr (void)
{
	static short fNew;
	static int j, w;
	static int bNrOfElements, bPattern, bPattern2;

	// Reload timer3, it is the morse code basic time unit;
	set_timer3(_morse_speed);

	switch (morse_state)
	{
	case MORSE_IDLE://0
		// Buffer Manager
		if(TX_Counter != 0)
		{
			if(fPrimeIt == 1)
			{
				fPrimeIt = 0;
				// test and wrap the pointer
				if(++TX_Rd_Index > TX_BUFFER_SIZE)
					TX_Rd_Index = 0;
				// keep track of the counter
				TX_Counter--;
			}
			if(TX_Counter != 0)
			{	// otherwise send char morse out
				TX_morse_char = TX_Buffer[TX_RD_Index];
				morse_state = MORSE_SEND;
				// test and wrap the pointer
				if(++TX_Rd_Index > TX_BUFFER_SIZE)
					TX_Rd_Index = 0;
				// keep track of the counter
				TX_Counter--;
			}

		}else{
			CW_send_active=FALSE;
		}
		break;
	case MORSE_SEND://1
		// Test range of ASCII input
		if((TX_morse_char >= 32) && (TX_morse_char <= 95))
			bPattern = u8_morse_LUT[TX_morse_char - ASCII_OFFSET];
		else
		{
			// special characters from case list :
			switch(TX_morse_char)
			{
			 case 'a': bPattern = u8_morse_LUT[AR]; break;  			 // 'AR' (.-.-.)
			 case 's': bPattern = u8_morse_LUT[SK]; break;   			 // 'SK' (...-.-)
			 case 'k': bPattern = u8_morse_LUT[KA]; break;   			 // 'KA' (-.-.-)
			 case 'n': bPattern = u8_morse_LUT[KN]; break;  				// 'KN' (-.--.)
			 case 'e': bPattern = u8_morse_LUT[EOM]; break;  			 // 'EOM' (.-----) used for "partitions" of msg
			 case 'c': bPattern = u8_morse_LUT[TX_COUNTER_ALIAS]; break; // 'C' transmit COUNTER alias serial number
			 case 'i': bPattern = u8_morse_LUT[I_COUNTER_ALIAS]; break;  // 'I' increment COUNTER alias serial number
			 case 't': bPattern = u8_morse_LUT[TUNE_TONE]; break;  		 // TUNE TONE #1 (31 dash times)
			 case 'u': bPattern = u8_morse_LUT[UNDERSTOOD]; break;		 // 'US' (...-.)
			 default:  bPattern = u8_morse_LUT[QUESTION];				 // anything else
			}
		}
		bPattern2 = bPattern;
		fNew = TRUE;
		morse_state = MORSE_XMIT;
		CW_send_active=TRUE;
#ifdef MORSE_DEBUG printf("\r\n %c: ", TX_morse_char); #endif
		break;
	case MORSE_XMIT://2
		// This section borrowed and modified for state machine use
		// from Wolfgang Buescher (DL4YHF) beacon program
		if(bPattern2 & 0x80) // bit 7 set -> SPACE or CONTROL character
		{
			bPattern2 &= 0x7F;			// reset space/control flag
			if(bPattern2 & 0x40)    	// bit6 of letter is control flag
			{   // bit7=1, bit6=1 bit5=x -> special functions ( 0b11Xn.nnnn )
				bPattern2 &= 0xBF;
				if(bPattern2 & 0x20) 	// 0b111X.nnnn = reserved for future use
				{
				}
				else           			// 0b110n.nnnnn = extra long dash (with key down!)
				{             			// max length = 31 (DASH TIMES)
					w = bPattern << 1;  // load bits0..5 of pause code into "dot counter"
					w += bPattern;      // dash time multiplied by THREE gives dot time
					morse_state = MORSE_TONE;
				}
			} // end if    < 0b11Xn.nnnn >
			else 	// < 0b10nn.nnnn > -> start a "long pause" time, length coded in 5 bit
			{   	// analyze 6-bit "pause" code (variable length, max = 63 (DOT TIMES)
				w = bPattern2 + 1;
				j=0;
				morse_state = MORSE_PAUSE;		// wait for long pause time
			} // end else  < 0b10Xn.nnnn >
		}
		else	// bit 7 not set -> normal character, neither SPACE nor CONTROL
		{		// play 'normal' letter :
			if(fNew)
			{
				bNrOfElements = 6;    // load number of dots+dashes ...
				while( !(bPattern & 0x40) )
				{
					bPattern <<= 1;
					bNrOfElements--;
					if(!bNrOfElements)
					{
						break;
					}
				}
				fNew = FALSE;
			}
			// Arrived here, bPattern holds the count of 'elements' in the pattern.
			// Bit 5 in bPattern holds the next element to be transmitted .
			if( bNrOfElements )
			{
				w = 1;            // guess it's a DOT
				if( bPattern & 0x20 )
				   w = 3;         // no, it's a DASH

				morse_state = MORSE_OUT;
				bPattern <<= 1;   // move next DASH / DOT bit in
				bNrOfElements--;
			}else{
				w = 2;
				morse_state = MORSE_END;
			}
			j=0;
		}
		break;
	case MORSE_OUT://3
		if(j < w)
		{
#ifdef MORSE_DEBUG printf("_"); #endif
			fTone_send = TRUE;
			//output_bit(TX_PTT, TONE);
		}else{
			morse_state = MORSE_SILENCE;
		}
		j++;
		break;
	case MORSE_TONE://4
			if(j < w)
			{
#ifdef MORSE_DEBUG printf("_"); #endif
				fTone_send = TRUE;
				//output_bit(TX_PTT, TONE);
			}else{
				fTone_send = FALSE;
				//output_bit(TX_PTT,NO_TONE);
				morse_state = MORSE_IDLE;
			}
			j++;
		break;
	case MORSE_PAUSE://6
		if(j < w)
		{
#ifdef MORSE_DEBUG printf(" "); #endif
			fTone_send = FALSE;
			//output_bit(TX_PTT, NO_TONE);
		}else{
			morse_state = MORSE_IDLE;
		}
		j++;
		break;
	case MORSE_SILENCE://5
#ifdef MORSE_DEBUG printf(" "); #endif
		fTone_send = FALSE;
		//output_bit(TX_PTT,NO_TONE);
		morse_state = MORSE_XMIT;
		break;
	case MORSE_END://4
		if(j < w)
		{
#ifdef MORSE_DEBUG printf(" "); #endif
			fTone_send = FALSE;
			//output_bit(TX_PTT, NO_TONE);
		}else{
			morse_state = MORSE_IDLE;
		}
		j++;
		break;
	default:
		morse_state = MORSE_IDLE;
		break;
	}
}







