
 /*****************************************************************************
 Calculating Morse Code Speed
 ============================
 The word PARIS is the standard for determing CW code speed. Each dit is one
 element, each dah is three elements, intra-character spacing is one
 element, inter-character spacing is three elements and inter-word spacing
 is seven elements. The word PARIS is exactly 50 elements.
 Note that after each dit/dah of the letter P -- one element spacing is used
 except the last one. (Intra-Character).

 After the last dit of P is sent, 3 elements are added (Inter-Character).
 After the word PARIS - 7 elements are used.

 Thus:
 P = di da da di = 1 1 3 1 3 1 1 (3) = 14 elements
 A = di da = 1 1 3 (3) = 8 elements
 R = di da di = 1 1 3 1 1 (3) = 10 elements
 I = di di = 1 1 1 (3) = 6 elements
 S = di di di = 1 1 1 1 1 [7] = 12 elements
 Total = 50 elements
 () = intercharacter
 [] = interword

 If you send PARIS 5 times in a minute (5WPM) you have sent 250 elements
 (using correct spacing). 250 elements into
 60 seconds per minute = 240 milliseconds per element.

 13 words-per-minute is one element every 92.31 milliseconds.

 DOT = 1U
 DASH = 3U
 SPACE BETWEEN COMPONENTS = 1U
 SPACE BETWEEN CHARACTERS = 3U
 SPACE BETWEEN WORDS = 7U

 Special Morse Code Characters:

 http://www.uncp.edu/home/acurtis/Courses/ResourcesForCourses/MorseCode.html

 Period ---
 Comma - -- -
 Slash --
 Plus --
 Equal --
 Question - -
 Open Paren -- -
 Close Paren -- --
 Dash --
 Quote --
 Underscore - --
 Single Quote - - - -
 Colon - - -
 Semicolon ---
 Dollar Sign --
 Warning --
 Error 
 Repetition(ii ii)  

*/


 // Storage format of "spaces", "control characters" and "transmittable CW letters"
 // ---------------------------------------------------------------------------------
 //  The most significant bits define the code type.
 //     Bit7: 1 = "this is SPACE (=pause) or CONTROL character", in this case:
 //        Bit6,Bit5 = control character type:
 //        0x = SPACE (=pause,     bits5..0 contain the length in DOTS (max.63)
 //        10 = extra long "dash", bits4..0 contain the length in DOTS (max.31)
 //        11 = future reserve,    bits4..0 contain 31 possible codes
 //     Bit7: 0 = "this is a transmittable character (not SPACE or CONTROL)".
 //        Bits6..0 contain a maximum of 6 dashes or dots,
 //             with leading 1="Startbit" before the "dash/dot-matrix",
 //             dash/dot-matrix is in Bit0..max.Bit5:
 //                  0=dot 1=dash
 //             Bit0 always contains the LAST TRANSMITTED dash/dot.
 // See definitions below for some examples how CW letters are stored in memory.

 // Morse code Lookup table
 int8 const u8_morse_LUT [] =
 {
	0b10000011,	// " "	32	Blank
	0b01101011,	// !	33	exclamation point (non-standard)
	0b01010010,	// " 	34 	Straight quote Mark
	0b10000011,	// #" "	35	Blank
	0b10000011,	// $" "	36	Dollar sign - Blank
	0b10000011,	// %" "	37	percentage  - Blank
	0b10000011,	// &" "	38	Ampersand   - Blank
	0b01011110,	// ' 	39 	Apostrophe
	0b00110110,	// ( 	40 	Left  parenthesis
	0b01101101,	// ) 	41 	Right parenthesis
	0b00011001,	// x	42	Multiplication sign
	0b00101010,	// +	43	Plus
	0b01110011,	// , 	44 	Comma
	0b01100001,	// - 	45 	Hyphen
	0b01010101,	// . 	46 	Period
	0b00110010,	// / 	47 	Fraction bar or division sign
	0b00111111,  // "0" 48
	0b00101111,  // "1"	49
	0b00100111,  // "2"	50
	0b00100011,  // "3"
	0b00100001,  // "4"
	0b00100000,  // "5"
	0b00110000,  // "6"
	0b00111000,  // "7"
	0b00111100,  // "8"
	0b00111110,  // "9"
	0b01111000,	 // :	58  Colon or division sign
	0b01101010,	 // ;	59	Semicolon
	0b10000011,	 // <	60	Blank
	0b00110001,	 // '='	61 	Double Hyphen
	0b10000011,	 // '>'	62	Blank
	0b01001100,	 // '?'	63 	Query
	0b01011010	 // '@'	64 	At sign (french "arobase") ITU added in 2004
	0b00000101,  // 'A'	65
	0b00011000,  // 'B'
	0b00011010,  // 'C'
	0b00001100,  // 'D'
	0b00000010,  // 'E'
	0b00010010,  // 'F'
	0b00001110,  // 'G'
	0b00010000,  // 'H'
	0b00000100,  // 'I'
	0b00010111,  // 'J'
	0b00001101,  // 'K'
	0b00010100,  // 'L'
	0b00000111,  // 'M'
	0b00000110,  // 'N'
	0b00001111,  // 'O'
	0b00010110,  // 'P'
	0b00011101,  // 'Q'
	0b00001010,  // 'R'
	0b00001000,  // 'S'
	0b00000011,  // 'T'
	0b00001001,  // 'U'
	0b00010001,  // 'V'
	0b00001011,  // 'W'
	0b00011001,  // 'X'
	0b00011011,  // 'Y'
	0b00011100,  // 'Z'
	0b10000011,	 // " "
	0b10000011,	 // " "
	0b10000011,	 // " "
	0b10000011,	 // " "
	0b10000011,	 // 	95	_ Underscore 95
	0b10000011,	 // " "	96 dummy
	0b00101010,  // 'AR' (.-.-.) or cross or addition sign (ITU)
	0b01000101,  // 'SK' (...-.-)
	0b00110101,  // 'KA' (-.-.-)
	0b00110110,  // 'KN' (-.--.)
	0b01011111,  // 'EOM' (.-----) used for "partitions" of msg
	0b01101010,  // 'NNN' (-.-.-.) replaced by serial number
	0b01011010,  // 'ANN' (.--.-.) Advance to Next Number
	0b11011111,  // 'TUNE' extra long "dash", bits4..0 contain the length
	0b00100010	 // 'UNDRSTOOD' (...-.)

};

#define TX_BUFFER_SIZE 191

#define ASCII_OFFSET 32
#define AR 97 - ASCII_OFFSET
#define SK 98 - ASCII_OFFSET
#define KA 99 - ASCII_OFFSET
#define KN 100 - ASCII_OFFSET
#define EOM 101 - ASCII_OFFSET
#define TX_COUNTER_ALIAS 102 - ASCII_OFFSET
#define I_COUNTER_ALIAS   103 - ASCII_OFFSET
#define TUNE_TONE 104 - ASCII_OFFSET
#define QUESTION '?' - ASCII_OFFSET
#define UNDERSTOOD 105 - ASCII_OFFSET

#define MORSE_SPEED_DEFAULT 40536//53036
#define NO_TONE 0
#define FINISH 0
#define MC_CONT 1
#define TONE 1

#ifndef TX_PTT
	#define TX_PTT PIN_E2
#endif


enum
{
	MORSE_IDLE,
	MORSE_SEND,
	MORSE_XMIT,
	MORSE_OUT,
	MORSE_TONE,
	MORSE_END,
	MORSE_SILENCE,
	MORSE_PAUSE,
	MORSE_CHAR_SPACE
};

char TX_morse_char=0;
int morse_state = MORSE_IDLE;
char TX_Buffer [TX_BUFFER_SIZE +1];	// Morse TX Buffer
int TX_Counter=0;
int TX_Rd_Index=0;
int TX_Wr_Index=0;
int fPrimeIt=0;
int CW_send_active=0;

short fTone_send = FALSE;

long _morse_speed;



void morsecode_service_interrupt (void);
void morse_initialize(void);
void morsecode_isr(void);
void put_morse(char c);
void calc_WordsPerMinute(long);
